# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

MRCAT_METADATA = Metadata(
    id="18ee3a555dc21091bd71575f0cabc1362de4ece7.boutiques",
    name="mrcat",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_MrcatConfigParamsDictNoTag = typing.TypedDict('_MrcatConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
MrcatConfigParamsDictTagged = typing.TypedDict('MrcatConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
MrcatConfigParamsDict = _MrcatConfigParamsDictNoTag | MrcatConfigParamsDictTagged


_MrcatParamsDictNoTag = typing.TypedDict('_MrcatParamsDictNoTag', {
    "axis": typing.NotRequired[int | None],
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrcatConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image1": InputPathType,
    "image2": list[InputPathType],
    "output": str,
})
MrcatParamsDictTagged = typing.TypedDict('MrcatParamsDictTagged', {
    "@type": typing.Literal["mrtrix/mrcat"],
    "axis": typing.NotRequired[int | None],
    "datatype": typing.NotRequired[str | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[MrcatConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "image1": InputPathType,
    "image2": list[InputPathType],
    "output": str,
})
MrcatParamsDict = _MrcatParamsDictNoTag | MrcatParamsDictTagged


def mrcat_config(
    key: str,
    value: str,
) -> MrcatConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def mrcat_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrcatConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def mrcat_config_cargs(
    params: MrcatConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class MrcatOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrcatParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image."""


def mrcat_params(
    image1: InputPathType,
    image2: list[InputPathType],
    output: str,
    axis: int | None = None,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrcatConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> MrcatParamsDictTagged:
    """
    Build parameters.
    
    Args:
        image1: the first input image.
        image2: additional input image(s).
        output: the output image.
        axis: specify axis along which concatenation should be performed. By\
            default, the program will use the last non-singleton, non-spatial axis\
            of any of the input images - in other words axis 3 or whichever axis\
            (greater than 3) of the input images has size greater than one.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/mrcat",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "image1": image1,
        "image2": image2,
        "output": output,
    }
    if axis is not None:
        params["axis"] = axis
    if datatype is not None:
        params["datatype"] = datatype
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def mrcat_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MrcatParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("axis", None) is not None:
        if not isinstance(params["axis"], int):
            raise StyxValidationError(f'`axis` has the wrong type: Received `{type(params.get("axis", None))}` expected `int | None`')
    if params.get("datatype", None) is not None:
        if not isinstance(params["datatype"], str):
            raise StyxValidationError(f'`datatype` has the wrong type: Received `{type(params.get("datatype", None))}` expected `str | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[MrcatConfigParamsDict] | None`')
        for e in params["config"]:
            mrcat_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("image1", None) is None:
        raise StyxValidationError("`image1` must not be None")
    if not isinstance(params["image1"], (pathlib.Path, str)):
        raise StyxValidationError(f'`image1` has the wrong type: Received `{type(params.get("image1", None))}` expected `InputPathType`')
    if params.get("image2", None) is None:
        raise StyxValidationError("`image2` must not be None")
    if not isinstance(params["image2"], list):
        raise StyxValidationError(f'`image2` has the wrong type: Received `{type(params.get("image2", None))}` expected `list[InputPathType]`')
    for e in params["image2"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`image2` has the wrong type: Received `{type(params.get("image2", None))}` expected `list[InputPathType]`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')


def mrcat_cargs(
    params: MrcatParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("mrcat")
    if params.get("axis", None) is not None:
        cargs.extend([
            "-axis",
            str(params.get("axis", None))
        ])
    if params.get("datatype", None) is not None:
        cargs.extend([
            "-datatype",
            params.get("datatype", None)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [mrcat_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("image1", None)))
    cargs.extend([execution.input_file(f) for f in params.get("image2", None)])
    cargs.append(params.get("output", None))
    return cargs


def mrcat_outputs(
    params: MrcatParamsDict,
    execution: Execution,
) -> MrcatOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MrcatOutputs(
        root=execution.output_file("."),
        output=execution.output_file(params.get("output", None)),
    )
    return ret


def mrcat_execute(
    params: MrcatParamsDict,
    runner: Runner | None = None,
) -> MrcatOutputs:
    """
    mrcat
    
    Concatenate several images into one.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrcatOutputs`).
    """
    mrcat_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRCAT_METADATA)
    params = execution.params(params)
    cargs = mrcat_cargs(params, execution)
    ret = mrcat_outputs(params, execution)
    execution.run(cargs)
    return ret


def mrcat(
    image1: InputPathType,
    image2: list[InputPathType],
    output: str,
    axis: int | None = None,
    datatype: str | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrcatConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrcatOutputs:
    """
    mrcat
    
    Concatenate several images into one.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        image1: the first input image.
        image2: additional input image(s).
        output: the output image.
        axis: specify axis along which concatenation should be performed. By\
            default, the program will use the last non-singleton, non-spatial axis\
            of any of the input images - in other words axis 3 or whichever axis\
            (greater than 3) of the input images has size greater than one.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrcatOutputs`).
    """
    params = mrcat_params(
        axis=axis,
        datatype=datatype,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        image1=image1,
        image2=image2,
        output=output,
    )
    return mrcat_execute(params, runner)


__all__ = [
    "MRCAT_METADATA",
    "MrcatConfigParamsDict",
    "MrcatConfigParamsDictTagged",
    "MrcatOutputs",
    "MrcatParamsDict",
    "MrcatParamsDictTagged",
    "mrcat",
    "mrcat_config",
    "mrcat_execute",
    "mrcat_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

AMP2SH_METADATA = Metadata(
    id="b1deae186542fe482c4a2e51622f124fe000c4fc.boutiques",
    name="amp2sh",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Amp2shFslgradParamsDictNoTag = typing.TypedDict('_Amp2shFslgradParamsDictNoTag', {
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Amp2shFslgradParamsDictTagged = typing.TypedDict('Amp2shFslgradParamsDictTagged', {
    "@type": typing.Literal["fslgrad"],
    "bvecs": InputPathType,
    "bvals": InputPathType,
})
Amp2shFslgradParamsDict = _Amp2shFslgradParamsDictNoTag | Amp2shFslgradParamsDictTagged


_Amp2shVariousStringParamsDictNoTag = typing.TypedDict('_Amp2shVariousStringParamsDictNoTag', {
    "obj": str,
})
Amp2shVariousStringParamsDictTagged = typing.TypedDict('Amp2shVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
Amp2shVariousStringParamsDict = _Amp2shVariousStringParamsDictNoTag | Amp2shVariousStringParamsDictTagged


_Amp2shVariousFileParamsDictNoTag = typing.TypedDict('_Amp2shVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
Amp2shVariousFileParamsDictTagged = typing.TypedDict('Amp2shVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
Amp2shVariousFileParamsDict = _Amp2shVariousFileParamsDictNoTag | Amp2shVariousFileParamsDictTagged


_Amp2shConfigParamsDictNoTag = typing.TypedDict('_Amp2shConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Amp2shConfigParamsDictTagged = typing.TypedDict('Amp2shConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Amp2shConfigParamsDict = _Amp2shConfigParamsDictNoTag | Amp2shConfigParamsDictTagged


_Amp2shParamsDictNoTag = typing.TypedDict('_Amp2shParamsDictNoTag', {
    "lmax": typing.NotRequired[int | None],
    "normalise": bool,
    "directions": typing.NotRequired[InputPathType | None],
    "rician": typing.NotRequired[InputPathType | None],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Amp2shFslgradParamsDict | None],
    "shells": typing.NotRequired[list[float] | None],
    "strides": typing.NotRequired[typing.Union[Amp2shVariousStringParamsDictTagged, Amp2shVariousFileParamsDictTagged] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Amp2shConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "amp": InputPathType,
    "SH": str,
})
Amp2shParamsDictTagged = typing.TypedDict('Amp2shParamsDictTagged', {
    "@type": typing.Literal["mrtrix/amp2sh"],
    "lmax": typing.NotRequired[int | None],
    "normalise": bool,
    "directions": typing.NotRequired[InputPathType | None],
    "rician": typing.NotRequired[InputPathType | None],
    "grad": typing.NotRequired[InputPathType | None],
    "fslgrad": typing.NotRequired[Amp2shFslgradParamsDict | None],
    "shells": typing.NotRequired[list[float] | None],
    "strides": typing.NotRequired[typing.Union[Amp2shVariousStringParamsDictTagged, Amp2shVariousFileParamsDictTagged] | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Amp2shConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "amp": InputPathType,
    "SH": str,
})
Amp2shParamsDict = _Amp2shParamsDictNoTag | Amp2shParamsDictTagged


def amp2sh_strides_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": amp2sh_various_string_cargs,
        "VariousFile": amp2sh_various_file_cargs,
    }.get(t)


def amp2sh_strides_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": amp2sh_various_string_validate,
        "VariousFile": amp2sh_various_file_validate,
    }.get(t)


def amp2sh_fslgrad(
    bvecs: InputPathType,
    bvals: InputPathType,
) -> Amp2shFslgradParamsDictTagged:
    """
    Build parameters.
    
    Args:
        bvecs: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        bvals: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "fslgrad",
        "bvecs": bvecs,
        "bvals": bvals,
    }
    return params


def amp2sh_fslgrad_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Amp2shFslgradParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("bvecs", None) is None:
        raise StyxValidationError("`bvecs` must not be None")
    if not isinstance(params["bvecs"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvecs` has the wrong type: Received `{type(params.get("bvecs", None))}` expected `InputPathType`')
    if params.get("bvals", None) is None:
        raise StyxValidationError("`bvals` must not be None")
    if not isinstance(params["bvals"], (pathlib.Path, str)):
        raise StyxValidationError(f'`bvals` has the wrong type: Received `{type(params.get("bvals", None))}` expected `InputPathType`')


def amp2sh_fslgrad_cargs(
    params: Amp2shFslgradParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-fslgrad")
    cargs.append(execution.input_file(params.get("bvecs", None)))
    cargs.append(execution.input_file(params.get("bvals", None)))
    return cargs


def amp2sh_various_string(
    obj: str,
) -> Amp2shVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def amp2sh_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Amp2shVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def amp2sh_various_string_cargs(
    params: Amp2shVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def amp2sh_various_file(
    obj: InputPathType,
) -> Amp2shVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def amp2sh_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Amp2shVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def amp2sh_various_file_cargs(
    params: Amp2shVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


def amp2sh_config(
    key: str,
    value: str,
) -> Amp2shConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def amp2sh_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Amp2shConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def amp2sh_config_cargs(
    params: Amp2shConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Amp2shOutputs(typing.NamedTuple):
    """
    Output object returned when calling `Amp2shParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    sh: OutputPathType
    """the output spherical harmonics coefficients image."""


def amp2sh_params(
    amp: InputPathType,
    sh: str,
    lmax: int | None = None,
    normalise: bool = False,
    directions: InputPathType | None = None,
    rician: InputPathType | None = None,
    grad: InputPathType | None = None,
    fslgrad: Amp2shFslgradParamsDict | None = None,
    shells: list[float] | None = None,
    strides: typing.Union[Amp2shVariousStringParamsDictTagged, Amp2shVariousFileParamsDictTagged] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Amp2shConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Amp2shParamsDictTagged:
    """
    Build parameters.
    
    Args:
        amp: the input amplitude image.
        sh: the output spherical harmonics coefficients image.
        lmax: set the maximum harmonic order for the output series. By default,\
            the program will use the highest possible lmax given the number of\
            diffusion-weighted images, up to a maximum of 8.
        normalise: normalise the DW signal to the b=0 image.
        directions: the directions corresponding to the input amplitude image\
            used to sample AFD. By default this option is not required providing\
            the direction set is supplied in the amplitude image. This should be\
            supplied as a list of directions [az el], as generated using the dirgen\
            command, or as a list of [ x y z ] Cartesian coordinates.
        rician: correct for Rician noise induced bias, using noise map supplied.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/amp2sh",
        "normalise": normalise,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "amp": amp,
        "SH": sh,
    }
    if lmax is not None:
        params["lmax"] = lmax
    if directions is not None:
        params["directions"] = directions
    if rician is not None:
        params["rician"] = rician
    if grad is not None:
        params["grad"] = grad
    if fslgrad is not None:
        params["fslgrad"] = fslgrad
    if shells is not None:
        params["shells"] = shells
    if strides is not None:
        params["strides"] = strides
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def amp2sh_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Amp2shParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("lmax", None) is not None:
        if not isinstance(params["lmax"], int):
            raise StyxValidationError(f'`lmax` has the wrong type: Received `{type(params.get("lmax", None))}` expected `int | None`')
    if params.get("normalise", False) is None:
        raise StyxValidationError("`normalise` must not be None")
    if not isinstance(params["normalise"], bool):
        raise StyxValidationError(f'`normalise` has the wrong type: Received `{type(params.get("normalise", False))}` expected `bool`')
    if params.get("directions", None) is not None:
        if not isinstance(params["directions"], (pathlib.Path, str)):
            raise StyxValidationError(f'`directions` has the wrong type: Received `{type(params.get("directions", None))}` expected `InputPathType | None`')
    if params.get("rician", None) is not None:
        if not isinstance(params["rician"], (pathlib.Path, str)):
            raise StyxValidationError(f'`rician` has the wrong type: Received `{type(params.get("rician", None))}` expected `InputPathType | None`')
    if params.get("grad", None) is not None:
        if not isinstance(params["grad"], (pathlib.Path, str)):
            raise StyxValidationError(f'`grad` has the wrong type: Received `{type(params.get("grad", None))}` expected `InputPathType | None`')
    if params.get("fslgrad", None) is not None:
        amp2sh_fslgrad_validate(params["fslgrad"])
    if params.get("shells", None) is not None:
        if not isinstance(params["shells"], list):
            raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
        for e in params["shells"]:
            if not isinstance(e, (float, int)):
                raise StyxValidationError(f'`shells` has the wrong type: Received `{type(params.get("shells", None))}` expected `list[float] | None`')
    if params.get("strides", None) is not None:
        if not isinstance(params["strides"], dict):
            raise StyxValidationError(f'Params object has the wrong type \'{type(params["strides"])}\'')
        if "@type" not in params["strides"]:
            raise StyxValidationError("Params object is missing `@type`")
        if params["strides"]["@type"] not in ["VariousString", "VariousFile"]:
            raise StyxValidationError("Parameter `strides`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
        amp2sh_strides_validate_dyn_fn(params["strides"]["@type"])(params["strides"])
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Amp2shConfigParamsDict] | None`')
        for e in params["config"]:
            amp2sh_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("amp", None) is None:
        raise StyxValidationError("`amp` must not be None")
    if not isinstance(params["amp"], (pathlib.Path, str)):
        raise StyxValidationError(f'`amp` has the wrong type: Received `{type(params.get("amp", None))}` expected `InputPathType`')
    if params.get("SH", None) is None:
        raise StyxValidationError("`SH` must not be None")
    if not isinstance(params["SH"], str):
        raise StyxValidationError(f'`SH` has the wrong type: Received `{type(params.get("SH", None))}` expected `str`')


def amp2sh_cargs(
    params: Amp2shParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("amp2sh")
    if params.get("lmax", None) is not None:
        cargs.extend([
            "-lmax",
            str(params.get("lmax", None))
        ])
    if params.get("normalise", False):
        cargs.append("-normalise")
    if params.get("directions", None) is not None:
        cargs.extend([
            "-directions",
            execution.input_file(params.get("directions", None))
        ])
    if params.get("rician", None) is not None:
        cargs.extend([
            "-rician",
            execution.input_file(params.get("rician", None))
        ])
    if params.get("grad", None) is not None:
        cargs.extend([
            "-grad",
            execution.input_file(params.get("grad", None))
        ])
    if params.get("fslgrad", None) is not None:
        cargs.extend(amp2sh_fslgrad_cargs(params.get("fslgrad", None), execution))
    if params.get("shells", None) is not None:
        cargs.extend([
            "-shells",
            ",".join(map(str, params.get("shells", None)))
        ])
    if params.get("strides", None) is not None:
        cargs.extend([
            "-strides",
            *amp2sh_strides_cargs_dyn_fn(params.get("strides", None)["@type"])(params.get("strides", None), execution)
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [amp2sh_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("amp", None)))
    cargs.append(params.get("SH", None))
    return cargs


def amp2sh_outputs(
    params: Amp2shParamsDict,
    execution: Execution,
) -> Amp2shOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Amp2shOutputs(
        root=execution.output_file("."),
        sh=execution.output_file(params.get("SH", None)),
    )
    return ret


def amp2sh_execute(
    params: Amp2shParamsDict,
    runner: Runner | None = None,
) -> Amp2shOutputs:
    """
    amp2sh
    
    Convert a set of amplitudes (defined along a set of corresponding
    directions) to their spherical harmonic representation.
    
    The spherical harmonic decomposition is calculated by least-squares linear
    fitting to the amplitude data.
    
    The directions can be defined either as a DW gradient scheme (for example to
    compute the SH representation of the DW signal), a set of [az el] pairs as
    output by the dirgen command, or a set of [ x y z ] directions in Cartesian
    coordinates. The DW gradient scheme or direction set can be supplied within
    the input image header or using the -gradient or -directions option. Note
    that if a direction set and DW gradient scheme can be found, the direction
    set will be used by default.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Amp2shOutputs`).
    """
    amp2sh_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(AMP2SH_METADATA)
    params = execution.params(params)
    cargs = amp2sh_cargs(params, execution)
    ret = amp2sh_outputs(params, execution)
    execution.run(cargs)
    return ret


def amp2sh(
    amp: InputPathType,
    sh: str,
    lmax: int | None = None,
    normalise: bool = False,
    directions: InputPathType | None = None,
    rician: InputPathType | None = None,
    grad: InputPathType | None = None,
    fslgrad: Amp2shFslgradParamsDict | None = None,
    shells: list[float] | None = None,
    strides: typing.Union[Amp2shVariousStringParamsDictTagged, Amp2shVariousFileParamsDictTagged] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Amp2shConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Amp2shOutputs:
    """
    amp2sh
    
    Convert a set of amplitudes (defined along a set of corresponding
    directions) to their spherical harmonic representation.
    
    The spherical harmonic decomposition is calculated by least-squares linear
    fitting to the amplitude data.
    
    The directions can be defined either as a DW gradient scheme (for example to
    compute the SH representation of the DW signal), a set of [az el] pairs as
    output by the dirgen command, or a set of [ x y z ] directions in Cartesian
    coordinates. The DW gradient scheme or direction set can be supplied within
    the input image header or using the -gradient or -directions option. Note
    that if a direction set and DW gradient scheme can be found, the direction
    set will be used by default.
    
    The spherical harmonic coefficients are stored according the conventions
    described the main documentation, which can be found at the following link:
    https://mrtrix.readthedocs.io/en/3.0.4/concepts/spherical_harmonics.html
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        amp: the input amplitude image.
        sh: the output spherical harmonics coefficients image.
        lmax: set the maximum harmonic order for the output series. By default,\
            the program will use the highest possible lmax given the number of\
            diffusion-weighted images, up to a maximum of 8.
        normalise: normalise the DW signal to the b=0 image.
        directions: the directions corresponding to the input amplitude image\
            used to sample AFD. By default this option is not required providing\
            the direction set is supplied in the amplitude image. This should be\
            supplied as a list of directions [az el], as generated using the dirgen\
            command, or as a list of [ x y z ] Cartesian coordinates.
        rician: correct for Rician noise induced bias, using noise map supplied.
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        shells: specify one or more b-values to use during processing, as a\
            comma-separated list of the desired approximate b-values (b-values are\
            clustered to allow for small deviations). Note that some commands are\
            incompatible with multiple b-values, and will report an error if more\
            than one b-value is provided.\
            WARNING: note that, even though the b=0 volumes are never referred\
            to as shells in the literature, they still have to be explicitly\
            included in the list of b-values as provided to the -shell option!\
            Several algorithms which include the b=0 volumes in their\
            computations may otherwise return an undesired result.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Amp2shOutputs`).
    """
    params = amp2sh_params(
        lmax=lmax,
        normalise=normalise,
        directions=directions,
        rician=rician,
        grad=grad,
        fslgrad=fslgrad,
        shells=shells,
        strides=strides,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        amp=amp,
        sh=sh,
    )
    return amp2sh_execute(params, runner)


__all__ = [
    "AMP2SH_METADATA",
    "Amp2shConfigParamsDict",
    "Amp2shConfigParamsDictTagged",
    "Amp2shFslgradParamsDict",
    "Amp2shFslgradParamsDictTagged",
    "Amp2shOutputs",
    "Amp2shParamsDict",
    "Amp2shParamsDictTagged",
    "Amp2shVariousFileParamsDict",
    "Amp2shVariousFileParamsDictTagged",
    "Amp2shVariousStringParamsDict",
    "Amp2shVariousStringParamsDictTagged",
    "amp2sh",
    "amp2sh_config",
    "amp2sh_execute",
    "amp2sh_fslgrad",
    "amp2sh_params",
    "amp2sh_various_file",
    "amp2sh_various_string",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TRANSFORMCOMPOSE_METADATA = Metadata(
    id="00dff5d1cb74f969e2ad40f5dbeaedf75a029491.boutiques",
    name="transformcompose",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_TransformcomposeConfigParamsDictNoTag = typing.TypedDict('_TransformcomposeConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
TransformcomposeConfigParamsDictTagged = typing.TypedDict('TransformcomposeConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
TransformcomposeConfigParamsDict = _TransformcomposeConfigParamsDictNoTag | TransformcomposeConfigParamsDictTagged


_TransformcomposeVariousStringParamsDictNoTag = typing.TypedDict('_TransformcomposeVariousStringParamsDictNoTag', {
    "obj": str,
})
TransformcomposeVariousStringParamsDictTagged = typing.TypedDict('TransformcomposeVariousStringParamsDictTagged', {
    "@type": typing.Literal["VariousString"],
    "obj": str,
})
TransformcomposeVariousStringParamsDict = _TransformcomposeVariousStringParamsDictNoTag | TransformcomposeVariousStringParamsDictTagged


_TransformcomposeVariousFileParamsDictNoTag = typing.TypedDict('_TransformcomposeVariousFileParamsDictNoTag', {
    "obj": InputPathType,
})
TransformcomposeVariousFileParamsDictTagged = typing.TypedDict('TransformcomposeVariousFileParamsDictTagged', {
    "@type": typing.Literal["VariousFile"],
    "obj": InputPathType,
})
TransformcomposeVariousFileParamsDict = _TransformcomposeVariousFileParamsDictNoTag | TransformcomposeVariousFileParamsDictTagged


_TransformcomposeParamsDictNoTag = typing.TypedDict('_TransformcomposeParamsDictNoTag', {
    "template": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TransformcomposeConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": list[InputPathType],
    "output": typing.Union[TransformcomposeVariousStringParamsDictTagged, TransformcomposeVariousFileParamsDictTagged],
})
TransformcomposeParamsDictTagged = typing.TypedDict('TransformcomposeParamsDictTagged', {
    "@type": typing.Literal["mrtrix/transformcompose"],
    "template": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[TransformcomposeConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "input": list[InputPathType],
    "output": typing.Union[TransformcomposeVariousStringParamsDictTagged, TransformcomposeVariousFileParamsDictTagged],
})
TransformcomposeParamsDict = _TransformcomposeParamsDictNoTag | TransformcomposeParamsDictTagged


def transformcompose_output_cargs_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "VariousString": transformcompose_various_string_cargs,
        "VariousFile": transformcompose_various_file_cargs,
    }.get(t)


def transformcompose_output_validate_dyn_fn(
    t: str,
) -> typing.Any:
    """
    Get validate params function by command type.
    
    Args:
        t: Command type.
    Returns:
        Validate params function.
    """
    return {
        "VariousString": transformcompose_various_string_validate,
        "VariousFile": transformcompose_various_file_validate,
    }.get(t)


def transformcompose_config(
    key: str,
    value: str,
) -> TransformcomposeConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def transformcompose_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TransformcomposeConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def transformcompose_config_cargs(
    params: TransformcomposeConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


def transformcompose_various_string(
    obj: str,
) -> TransformcomposeVariousStringParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: String object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousString",
        "obj": obj,
    }
    return params


def transformcompose_various_string_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TransformcomposeVariousStringParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], str):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `str`')


def transformcompose_various_string_cargs(
    params: TransformcomposeVariousStringParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(params.get("obj", None))
    return cargs


def transformcompose_various_file(
    obj: InputPathType,
) -> TransformcomposeVariousFileParamsDictTagged:
    """
    Build parameters.
    
    Args:
        obj: File object.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "VariousFile",
        "obj": obj,
    }
    return params


def transformcompose_various_file_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TransformcomposeVariousFileParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("obj", None) is None:
        raise StyxValidationError("`obj` must not be None")
    if not isinstance(params["obj"], (pathlib.Path, str)):
        raise StyxValidationError(f'`obj` has the wrong type: Received `{type(params.get("obj", None))}` expected `InputPathType`')


def transformcompose_various_file_cargs(
    params: TransformcomposeVariousFileParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append(execution.input_file(params.get("obj", None)))
    return cargs


class TransformcomposeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `TransformcomposeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def transformcompose_params(
    input_: list[InputPathType],
    output: typing.Union[TransformcomposeVariousStringParamsDictTagged, TransformcomposeVariousFileParamsDictTagged],
    template: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TransformcomposeConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> TransformcomposeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input transforms (either linear or non-linear warps).
        output: the output file (may be a linear transformation text file, or a\
            deformation warp field image, depending on usage).
        template: define the output grid defined by a template image.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/transformcompose",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "input": input_,
        "output": output,
    }
    if template is not None:
        params["template"] = template
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def transformcompose_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `TransformcomposeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("template", None) is not None:
        if not isinstance(params["template"], (pathlib.Path, str)):
            raise StyxValidationError(f'`template` has the wrong type: Received `{type(params.get("template", None))}` expected `InputPathType | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[TransformcomposeConfigParamsDict] | None`')
        for e in params["config"]:
            transformcompose_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], list):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `list[InputPathType]`')
    for e in params["input"]:
        if not isinstance(e, (pathlib.Path, str)):
            raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `list[InputPathType]`')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params["output"])}\'')
    if "@type" not in params["output"]:
        raise StyxValidationError("Params object is missing `@type`")
    if params["output"]["@type"] not in ["VariousString", "VariousFile"]:
        raise StyxValidationError("Parameter `output`s `@type` must be one of [\"VariousString\", \"VariousFile\"]")
    transformcompose_output_validate_dyn_fn(params["output"]["@type"])(params["output"])


def transformcompose_cargs(
    params: TransformcomposeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("transformcompose")
    if params.get("template", None) is not None:
        cargs.extend([
            "-template",
            execution.input_file(params.get("template", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [transformcompose_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.extend([execution.input_file(f) for f in params.get("input", None)])
    cargs.extend(transformcompose_output_cargs_dyn_fn(params.get("output", None)["@type"])(params.get("output", None), execution))
    return cargs


def transformcompose_outputs(
    params: TransformcomposeParamsDict,
    execution: Execution,
) -> TransformcomposeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = TransformcomposeOutputs(
        root=execution.output_file("."),
    )
    return ret


def transformcompose_execute(
    params: TransformcomposeParamsDict,
    runner: Runner | None = None,
) -> TransformcomposeOutputs:
    """
    transformcompose
    
    Compose any number of linear transformations and/or warps into a single
    transformation.
    
    Any linear transforms must be supplied as a 4x4 matrix in a text file (e.g.
    as per the output of mrregister). Any warp fields must be supplied as a 4D
    image representing a deformation field (e.g. as output from mrrregister
    -nl_warp).
    
    Input transformations should be provided to the command in the order in
    which they would be applied to an image if they were to be applied
    individually.
    
    If all input transformations are linear, and the -template option is not
    provided, then the file output by the command will also be a linear
    transformation saved as a 4x4 matrix in a text file. If a template image is
    supplied, then the output will always be a deformation field. If at least
    one of the inputs is a warp field, then the output will be a deformation
    field, which will be defined on the grid of the last input warp image
    supplied if the -template option is not used.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TransformcomposeOutputs`).
    """
    transformcompose_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TRANSFORMCOMPOSE_METADATA)
    params = execution.params(params)
    cargs = transformcompose_cargs(params, execution)
    ret = transformcompose_outputs(params, execution)
    execution.run(cargs)
    return ret


def transformcompose(
    input_: list[InputPathType],
    output: typing.Union[TransformcomposeVariousStringParamsDictTagged, TransformcomposeVariousFileParamsDictTagged],
    template: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[TransformcomposeConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> TransformcomposeOutputs:
    """
    transformcompose
    
    Compose any number of linear transformations and/or warps into a single
    transformation.
    
    Any linear transforms must be supplied as a 4x4 matrix in a text file (e.g.
    as per the output of mrregister). Any warp fields must be supplied as a 4D
    image representing a deformation field (e.g. as output from mrrregister
    -nl_warp).
    
    Input transformations should be provided to the command in the order in
    which they would be applied to an image if they were to be applied
    individually.
    
    If all input transformations are linear, and the -template option is not
    provided, then the file output by the command will also be a linear
    transformation saved as a 4x4 matrix in a text file. If a template image is
    supplied, then the output will always be a deformation field. If at least
    one of the inputs is a warp field, then the output will be a deformation
    field, which will be defined on the grid of the last input warp image
    supplied if the -template option is not used.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: the input transforms (either linear or non-linear warps).
        output: the output file (may be a linear transformation text file, or a\
            deformation warp field image, depending on usage).
        template: define the output grid defined by a template image.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TransformcomposeOutputs`).
    """
    params = transformcompose_params(
        template=template,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        input_=input_,
        output=output,
    )
    return transformcompose_execute(params, runner)


__all__ = [
    "TRANSFORMCOMPOSE_METADATA",
    "TransformcomposeConfigParamsDict",
    "TransformcomposeConfigParamsDictTagged",
    "TransformcomposeOutputs",
    "TransformcomposeParamsDict",
    "TransformcomposeParamsDictTagged",
    "TransformcomposeVariousFileParamsDict",
    "TransformcomposeVariousFileParamsDictTagged",
    "TransformcomposeVariousStringParamsDict",
    "TransformcomposeVariousStringParamsDictTagged",
    "transformcompose",
    "transformcompose_config",
    "transformcompose_execute",
    "transformcompose_params",
    "transformcompose_various_file",
    "transformcompose_various_string",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

FIXELCONNECTIVITY_METADATA = Metadata(
    id="00a3b63e9ba975a20fec13986503b3adc2b7f0f5.boutiques",
    name="fixelconnectivity",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_FixelconnectivityConfigParamsDictNoTag = typing.TypedDict('_FixelconnectivityConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
FixelconnectivityConfigParamsDictTagged = typing.TypedDict('FixelconnectivityConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
FixelconnectivityConfigParamsDict = _FixelconnectivityConfigParamsDictNoTag | FixelconnectivityConfigParamsDictTagged


_FixelconnectivityParamsDictNoTag = typing.TypedDict('_FixelconnectivityParamsDictNoTag', {
    "threshold": typing.NotRequired[float | None],
    "angle": typing.NotRequired[float | None],
    "mask": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[FixelconnectivityConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "fixel_directory": InputPathType,
    "tracks": InputPathType,
    "matrix": str,
})
FixelconnectivityParamsDictTagged = typing.TypedDict('FixelconnectivityParamsDictTagged', {
    "@type": typing.Literal["mrtrix/fixelconnectivity"],
    "threshold": typing.NotRequired[float | None],
    "angle": typing.NotRequired[float | None],
    "mask": typing.NotRequired[InputPathType | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[FixelconnectivityConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "fixel_directory": InputPathType,
    "tracks": InputPathType,
    "matrix": str,
})
FixelconnectivityParamsDict = _FixelconnectivityParamsDictNoTag | FixelconnectivityParamsDictTagged


def fixelconnectivity_config(
    key: str,
    value: str,
) -> FixelconnectivityConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def fixelconnectivity_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelconnectivityConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def fixelconnectivity_config_cargs(
    params: FixelconnectivityConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class FixelconnectivityOutputs(typing.NamedTuple):
    """
    Output object returned when calling `FixelconnectivityParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    matrix: OutputPathType
    """the output fixel-fixel connectivity matrix directory path"""


def fixelconnectivity_params(
    fixel_directory: InputPathType,
    tracks: InputPathType,
    matrix: str,
    threshold: float | None = None,
    angle: float | None = None,
    mask: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelconnectivityConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> FixelconnectivityParamsDictTagged:
    """
    Build parameters.
    
    Args:
        fixel_directory: the directory containing the fixels between which\
            connectivity will be quantified.
        tracks: the tracks used to determine fixel-fixel connectivity.
        matrix: the output fixel-fixel connectivity matrix directory path.
        threshold: a threshold to define the required fraction of shared\
            connections to be included in the neighbourhood (default: 0.01).
        angle: the max angle threshold for assigning streamline tangents to\
            fixels (Default: 45 degrees).
        mask: provide a fixel data file containing a mask of those fixels to be\
            computed; fixels outside the mask will be empty in the output matrix.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/fixelconnectivity",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "fixel_directory": fixel_directory,
        "tracks": tracks,
        "matrix": matrix,
    }
    if threshold is not None:
        params["threshold"] = threshold
    if angle is not None:
        params["angle"] = angle
    if mask is not None:
        params["mask"] = mask
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def fixelconnectivity_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `FixelconnectivityParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("threshold", None) is not None:
        if not isinstance(params["threshold"], (float, int)):
            raise StyxValidationError(f'`threshold` has the wrong type: Received `{type(params.get("threshold", None))}` expected `float | None`')
    if params.get("angle", None) is not None:
        if not isinstance(params["angle"], (float, int)):
            raise StyxValidationError(f'`angle` has the wrong type: Received `{type(params.get("angle", None))}` expected `float | None`')
    if params.get("mask", None) is not None:
        if not isinstance(params["mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`mask` has the wrong type: Received `{type(params.get("mask", None))}` expected `InputPathType | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[FixelconnectivityConfigParamsDict] | None`')
        for e in params["config"]:
            fixelconnectivity_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("fixel_directory", None) is None:
        raise StyxValidationError("`fixel_directory` must not be None")
    if not isinstance(params["fixel_directory"], (pathlib.Path, str)):
        raise StyxValidationError(f'`fixel_directory` has the wrong type: Received `{type(params.get("fixel_directory", None))}` expected `InputPathType`')
    if params.get("tracks", None) is None:
        raise StyxValidationError("`tracks` must not be None")
    if not isinstance(params["tracks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`tracks` has the wrong type: Received `{type(params.get("tracks", None))}` expected `InputPathType`')
    if params.get("matrix", None) is None:
        raise StyxValidationError("`matrix` must not be None")
    if not isinstance(params["matrix"], str):
        raise StyxValidationError(f'`matrix` has the wrong type: Received `{type(params.get("matrix", None))}` expected `str`')


def fixelconnectivity_cargs(
    params: FixelconnectivityParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("fixelconnectivity")
    if params.get("threshold", None) is not None:
        cargs.extend([
            "-threshold",
            str(params.get("threshold", None))
        ])
    if params.get("angle", None) is not None:
        cargs.extend([
            "-angle",
            str(params.get("angle", None))
        ])
    if params.get("mask", None) is not None:
        cargs.extend([
            "-mask",
            execution.input_file(params.get("mask", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [fixelconnectivity_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("fixel_directory", None)))
    cargs.append(execution.input_file(params.get("tracks", None)))
    cargs.append(params.get("matrix", None))
    return cargs


def fixelconnectivity_outputs(
    params: FixelconnectivityParamsDict,
    execution: Execution,
) -> FixelconnectivityOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = FixelconnectivityOutputs(
        root=execution.output_file("."),
        matrix=execution.output_file(params.get("matrix", None)),
    )
    return ret


def fixelconnectivity_execute(
    params: FixelconnectivityParamsDict,
    runner: Runner | None = None,
) -> FixelconnectivityOutputs:
    """
    fixelconnectivity
    
    Generate a fixel-fixel connectivity matrix.
    
    This command will generate a directory containing three images, which
    encodes the fixel-fixel connectivity matrix. Documentation regarding this
    format and how to use it will come in the future.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FixelconnectivityOutputs`).
    """
    fixelconnectivity_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(FIXELCONNECTIVITY_METADATA)
    params = execution.params(params)
    cargs = fixelconnectivity_cargs(params, execution)
    ret = fixelconnectivity_outputs(params, execution)
    execution.run(cargs)
    return ret


def fixelconnectivity(
    fixel_directory: InputPathType,
    tracks: InputPathType,
    matrix: str,
    threshold: float | None = None,
    angle: float | None = None,
    mask: InputPathType | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[FixelconnectivityConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> FixelconnectivityOutputs:
    """
    fixelconnectivity
    
    Generate a fixel-fixel connectivity matrix.
    
    This command will generate a directory containing three images, which
    encodes the fixel-fixel connectivity matrix. Documentation regarding this
    format and how to use it will come in the future.
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        fixel_directory: the directory containing the fixels between which\
            connectivity will be quantified.
        tracks: the tracks used to determine fixel-fixel connectivity.
        matrix: the output fixel-fixel connectivity matrix directory path.
        threshold: a threshold to define the required fraction of shared\
            connections to be included in the neighbourhood (default: 0.01).
        angle: the max angle threshold for assigning streamline tangents to\
            fixels (Default: 45 degrees).
        mask: provide a fixel data file containing a mask of those fixels to be\
            computed; fixels outside the mask will be empty in the output matrix.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FixelconnectivityOutputs`).
    """
    params = fixelconnectivity_params(
        threshold=threshold,
        angle=angle,
        mask=mask,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        fixel_directory=fixel_directory,
        tracks=tracks,
        matrix=matrix,
    )
    return fixelconnectivity_execute(params, runner)


__all__ = [
    "FIXELCONNECTIVITY_METADATA",
    "FixelconnectivityConfigParamsDict",
    "FixelconnectivityConfigParamsDictTagged",
    "FixelconnectivityOutputs",
    "FixelconnectivityParamsDict",
    "FixelconnectivityParamsDictTagged",
    "fixelconnectivity",
    "fixelconnectivity_config",
    "fixelconnectivity_execute",
    "fixelconnectivity_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

DIRMERGE_METADATA = Metadata(
    id="40d07b2a5fb8d4016b42c7968c68cb8ea4d70a1a.boutiques",
    name="dirmerge",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_DirmergeConfigParamsDictNoTag = typing.TypedDict('_DirmergeConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
DirmergeConfigParamsDictTagged = typing.TypedDict('DirmergeConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
DirmergeConfigParamsDict = _DirmergeConfigParamsDictNoTag | DirmergeConfigParamsDictTagged


_DirmergeParamsDictNoTag = typing.TypedDict('_DirmergeParamsDictNoTag', {
    "unipolar_weight": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DirmergeConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "subsets": int,
    "bvalue_files": list[str],
    "out": str,
})
DirmergeParamsDictTagged = typing.TypedDict('DirmergeParamsDictTagged', {
    "@type": typing.Literal["mrtrix/dirmerge"],
    "unipolar_weight": typing.NotRequired[float | None],
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[DirmergeConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "subsets": int,
    "bvalue_files": list[str],
    "out": str,
})
DirmergeParamsDict = _DirmergeParamsDictNoTag | DirmergeParamsDictTagged


def dirmerge_config(
    key: str,
    value: str,
) -> DirmergeConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def dirmerge_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DirmergeConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def dirmerge_config_cargs(
    params: DirmergeConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class DirmergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `DirmergeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out: OutputPathType
    """the output directions file, with each row listing the X Y Z gradient
    directions, the b-value, and an index representing the phase encode
    direction"""


def dirmerge_params(
    subsets: int,
    bvalue_files: list[str],
    out: str,
    unipolar_weight: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DirmergeConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> DirmergeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        subsets: the number of subsets (eg. phase encoding directions) per\
            b-value.
        bvalue_files: the b-value and sets of corresponding files, in order.
        out: the output directions file, with each row listing the X Y Z\
            gradient directions, the b-value, and an index representing the phase\
            encode direction.
        unipolar_weight: set the weight given to the unipolar electrostatic\
            repulsion model compared to the bipolar model (default: 0.2).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/dirmerge",
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "subsets": subsets,
        "bvalue_files": bvalue_files,
        "out": out,
    }
    if unipolar_weight is not None:
        params["unipolar_weight"] = unipolar_weight
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def dirmerge_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `DirmergeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("unipolar_weight", None) is not None:
        if not isinstance(params["unipolar_weight"], (float, int)):
            raise StyxValidationError(f'`unipolar_weight` has the wrong type: Received `{type(params.get("unipolar_weight", None))}` expected `float | None`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[DirmergeConfigParamsDict] | None`')
        for e in params["config"]:
            dirmerge_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("subsets", None) is None:
        raise StyxValidationError("`subsets` must not be None")
    if not isinstance(params["subsets"], int):
        raise StyxValidationError(f'`subsets` has the wrong type: Received `{type(params.get("subsets", None))}` expected `int`')
    if params.get("bvalue_files", None) is None:
        raise StyxValidationError("`bvalue_files` must not be None")
    if not isinstance(params["bvalue_files"], list):
        raise StyxValidationError(f'`bvalue_files` has the wrong type: Received `{type(params.get("bvalue_files", None))}` expected `list[str]`')
    for e in params["bvalue_files"]:
        if not isinstance(e, str):
            raise StyxValidationError(f'`bvalue_files` has the wrong type: Received `{type(params.get("bvalue_files", None))}` expected `list[str]`')
    if params.get("out", None) is None:
        raise StyxValidationError("`out` must not be None")
    if not isinstance(params["out"], str):
        raise StyxValidationError(f'`out` has the wrong type: Received `{type(params.get("out", None))}` expected `str`')


def dirmerge_cargs(
    params: DirmergeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("dirmerge")
    if params.get("unipolar_weight", None) is not None:
        cargs.extend([
            "-unipolar_weight",
            str(params.get("unipolar_weight", None))
        ])
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [dirmerge_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(str(params.get("subsets", None)))
    cargs.extend(params.get("bvalue_files", None))
    cargs.append(params.get("out", None))
    return cargs


def dirmerge_outputs(
    params: DirmergeParamsDict,
    execution: Execution,
) -> DirmergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = DirmergeOutputs(
        root=execution.output_file("."),
        out=execution.output_file(params.get("out", None)),
    )
    return ret


def dirmerge_execute(
    params: DirmergeParamsDict,
    runner: Runner | None = None,
) -> DirmergeOutputs:
    """
    dirmerge
    
    Splice / merge multiple sets of directions in such a way as to maintain
    near-optimality upon truncation.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DirmergeOutputs`).
    """
    dirmerge_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(DIRMERGE_METADATA)
    params = execution.params(params)
    cargs = dirmerge_cargs(params, execution)
    ret = dirmerge_outputs(params, execution)
    execution.run(cargs)
    return ret


def dirmerge(
    subsets: int,
    bvalue_files: list[str],
    out: str,
    unipolar_weight: float | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[DirmergeConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> DirmergeOutputs:
    """
    dirmerge
    
    Splice / merge multiple sets of directions in such a way as to maintain
    near-optimality upon truncation.
    
    
    
    References:
    
    .
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        subsets: the number of subsets (eg. phase encoding directions) per\
            b-value.
        bvalue_files: the b-value and sets of corresponding files, in order.
        out: the output directions file, with each row listing the X Y Z\
            gradient directions, the b-value, and an index representing the phase\
            encode direction.
        unipolar_weight: set the weight given to the unipolar electrostatic\
            repulsion model compared to the bipolar model (default: 0.2).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DirmergeOutputs`).
    """
    params = dirmerge_params(
        unipolar_weight=unipolar_weight,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        subsets=subsets,
        bvalue_files=bvalue_files,
        out=out,
    )
    return dirmerge_execute(params, runner)


__all__ = [
    "DIRMERGE_METADATA",
    "DirmergeConfigParamsDict",
    "DirmergeConfigParamsDictTagged",
    "DirmergeOutputs",
    "DirmergeParamsDict",
    "DirmergeParamsDictTagged",
    "dirmerge",
    "dirmerge_config",
    "dirmerge_execute",
    "dirmerge_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

TCKSIFT2_METADATA = Metadata(
    id="14aaf15fab77274145a9c252945d1e6493faa9d5.boutiques",
    name="tcksift2",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


_Tcksift2ConfigParamsDictNoTag = typing.TypedDict('_Tcksift2ConfigParamsDictNoTag', {
    "key": str,
    "value": str,
})
Tcksift2ConfigParamsDictTagged = typing.TypedDict('Tcksift2ConfigParamsDictTagged', {
    "@type": typing.Literal["config"],
    "key": str,
    "value": str,
})
Tcksift2ConfigParamsDict = _Tcksift2ConfigParamsDictNoTag | Tcksift2ConfigParamsDictTagged


_Tcksift2ParamsDictNoTag = typing.TypedDict('_Tcksift2ParamsDictNoTag', {
    "proc_mask": typing.NotRequired[InputPathType | None],
    "act": typing.NotRequired[InputPathType | None],
    "fd_scale_gm": bool,
    "no_dilate_lut": bool,
    "make_null_lobes": bool,
    "remove_untracked": bool,
    "fd_thresh": typing.NotRequired[float | None],
    "csv": typing.NotRequired[str | None],
    "out_mu": typing.NotRequired[str | None],
    "output_debug": bool,
    "out_coeffs": typing.NotRequired[str | None],
    "reg_tikhonov": typing.NotRequired[float | None],
    "reg_tv": typing.NotRequired[float | None],
    "min_td_frac": typing.NotRequired[float | None],
    "min_iters": typing.NotRequired[int | None],
    "max_iters": typing.NotRequired[int | None],
    "min_factor": typing.NotRequired[float | None],
    "min_coeff": typing.NotRequired[float | None],
    "max_factor": typing.NotRequired[float | None],
    "max_coeff": typing.NotRequired[float | None],
    "max_coeff_step": typing.NotRequired[float | None],
    "min_cf_decrease": typing.NotRequired[float | None],
    "linear": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Tcksift2ConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in_tracks": InputPathType,
    "in_fod": InputPathType,
    "out_weights": str,
})
Tcksift2ParamsDictTagged = typing.TypedDict('Tcksift2ParamsDictTagged', {
    "@type": typing.Literal["mrtrix/tcksift2"],
    "proc_mask": typing.NotRequired[InputPathType | None],
    "act": typing.NotRequired[InputPathType | None],
    "fd_scale_gm": bool,
    "no_dilate_lut": bool,
    "make_null_lobes": bool,
    "remove_untracked": bool,
    "fd_thresh": typing.NotRequired[float | None],
    "csv": typing.NotRequired[str | None],
    "out_mu": typing.NotRequired[str | None],
    "output_debug": bool,
    "out_coeffs": typing.NotRequired[str | None],
    "reg_tikhonov": typing.NotRequired[float | None],
    "reg_tv": typing.NotRequired[float | None],
    "min_td_frac": typing.NotRequired[float | None],
    "min_iters": typing.NotRequired[int | None],
    "max_iters": typing.NotRequired[int | None],
    "min_factor": typing.NotRequired[float | None],
    "min_coeff": typing.NotRequired[float | None],
    "max_factor": typing.NotRequired[float | None],
    "max_coeff": typing.NotRequired[float | None],
    "max_coeff_step": typing.NotRequired[float | None],
    "min_cf_decrease": typing.NotRequired[float | None],
    "linear": bool,
    "info": bool,
    "quiet": bool,
    "debug": bool,
    "force": bool,
    "nthreads": typing.NotRequired[int | None],
    "config": typing.NotRequired[list[Tcksift2ConfigParamsDict] | None],
    "help": bool,
    "version": bool,
    "in_tracks": InputPathType,
    "in_fod": InputPathType,
    "out_weights": str,
})
Tcksift2ParamsDict = _Tcksift2ParamsDictNoTag | Tcksift2ParamsDictTagged


def tcksift2_config(
    key: str,
    value: str,
) -> Tcksift2ConfigParamsDictTagged:
    """
    Build parameters.
    
    Args:
        key: temporarily set the value of an MRtrix config file entry.
        value: temporarily set the value of an MRtrix config file entry.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "config",
        "key": key,
        "value": value,
    }
    return params


def tcksift2_config_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Tcksift2ConfigParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("key", None) is None:
        raise StyxValidationError("`key` must not be None")
    if not isinstance(params["key"], str):
        raise StyxValidationError(f'`key` has the wrong type: Received `{type(params.get("key", None))}` expected `str`')
    if params.get("value", None) is None:
        raise StyxValidationError("`value` must not be None")
    if not isinstance(params["value"], str):
        raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `str`')


def tcksift2_config_cargs(
    params: Tcksift2ConfigParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-config")
    cargs.append(params.get("key", None))
    cargs.append(params.get("value", None))
    return cargs


class Tcksift2Outputs(typing.NamedTuple):
    """
    Output object returned when calling `Tcksift2ParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_weights: OutputPathType
    """output text file containing the weighting factor for each streamline"""
    csv_: OutputPathType | None
    """output statistics of execution per iteration to a .csv file """
    out_mu: OutputPathType | None
    """output the final value of SIFT proportionality coefficient mu to a text
    file """
    out_coeffs: OutputPathType | None
    """output text file containing the weighting coefficient for each streamline
    """


def tcksift2_params(
    in_tracks: InputPathType,
    in_fod: InputPathType,
    out_weights: str,
    proc_mask: InputPathType | None = None,
    act: InputPathType | None = None,
    fd_scale_gm: bool = False,
    no_dilate_lut: bool = False,
    make_null_lobes: bool = False,
    remove_untracked: bool = False,
    fd_thresh: float | None = None,
    csv_: str | None = None,
    out_mu: str | None = None,
    output_debug: bool = False,
    out_coeffs: str | None = None,
    reg_tikhonov: float | None = None,
    reg_tv: float | None = None,
    min_td_frac: float | None = None,
    min_iters: int | None = None,
    max_iters: int | None = None,
    min_factor: float | None = None,
    min_coeff: float | None = None,
    max_factor: float | None = None,
    max_coeff: float | None = None,
    max_coeff_step: float | None = None,
    min_cf_decrease: float | None = None,
    linear: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Tcksift2ConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
) -> Tcksift2ParamsDictTagged:
    """
    Build parameters.
    
    Args:
        in_tracks: the input track file.
        in_fod: input image containing the spherical harmonics of the fibre\
            orientation distributions.
        out_weights: output text file containing the weighting factor for each\
            streamline.
        proc_mask: provide an image containing the processing mask weights for\
            the model; image spatial dimensions must match the fixel image.
        act: use an ACT five-tissue-type segmented anatomical image to derive\
            the processing mask.
        fd_scale_gm: provide this option (in conjunction with -act) to\
            heuristically downsize the fibre density estimates based on the\
            presence of GM in the voxel. This can assist in reducing tissue\
            interface effects when using a single-tissue deconvolution algorithm.
        no_dilate_lut: do NOT dilate FOD lobe lookup tables; only map\
            streamlines to FOD lobes if the precise tangent lies within the angular\
            spread of that lobe.
        make_null_lobes: add an additional FOD lobe to each voxel, with zero\
            integral, that covers all directions with zero / negative FOD\
            amplitudes.
        remove_untracked: remove FOD lobes that do not have any streamline\
            density attributed to them; this improves filtering slightly, at the\
            expense of longer computation time (and you can no longer do\
            quantitative comparisons between reconstructions if this is enabled).
        fd_thresh: fibre density threshold; exclude an FOD lobe from filtering\
            processing if its integral is less than this amount (streamlines will\
            still be mapped to it, but it will not contribute to the cost function\
            or the filtering).
        csv_: output statistics of execution per iteration to a .csv file.
        out_mu: output the final value of SIFT proportionality coefficient mu\
            to a text file.
        output_debug: provide various output images for assessing & debugging\
            performance etc.
        out_coeffs: output text file containing the weighting coefficient for\
            each streamline.
        reg_tikhonov: provide coefficient for regularising streamline weighting\
            coefficients (Tikhonov regularisation) (default: 0).
        reg_tv: provide coefficient for regularising variance of streamline\
            weighting coefficient to fixels along its length (Total Variation\
            regularisation) (default: 0.1).
        min_td_frac: minimum fraction of the FOD integral reconstructed by\
            streamlines; if the reconstructed streamline density is below this\
            fraction, the fixel is excluded from optimisation (default: 0.1).
        min_iters: minimum number of iterations to run before testing for\
            convergence; this can prevent premature termination at early iterations\
            if the cost function increases slightly (default: 10).
        max_iters: maximum number of iterations to run before terminating\
            program.
        min_factor: minimum weighting factor for an individual streamline; if\
            the factor falls below this number the streamline will be rejected\
            entirely (factor set to zero) (default: 0).
        min_coeff: minimum weighting coefficient for an individual streamline;\
            similar to the '-min_factor' option, but using the exponential\
            coefficient basis of the SIFT2 model; these parameters are related as:\
            factor = e^(coeff). Note that the -min_factor and -min_coeff options\
            are mutually exclusive - you can only provide one. (default: -inf).
        max_factor: maximum weighting factor that can be assigned to any one\
            streamline (default: inf).
        max_coeff: maximum weighting coefficient for an individual streamline;\
            similar to the '-max_factor' option, but using the exponential\
            coefficient basis of the SIFT2 model; these parameters are related as:\
            factor = e^(coeff). Note that the -max_factor and -max_coeff options\
            are mutually exclusive - you can only provide one. (default: inf).
        max_coeff_step: maximum change to a streamline's weighting coefficient\
            in a single iteration (default: 1).
        min_cf_decrease: minimum decrease in the cost function (as a fraction\
            of the initial value) that must occur each iteration for the algorithm\
            to continue (default: 2.5e-05).
        linear: perform a linear estimation of streamline weights, rather than\
            the standard non-linear optimisation (typically does not provide as\
            accurate a model fit; but only requires a single pass).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "mrtrix/tcksift2",
        "fd_scale_gm": fd_scale_gm,
        "no_dilate_lut": no_dilate_lut,
        "make_null_lobes": make_null_lobes,
        "remove_untracked": remove_untracked,
        "output_debug": output_debug,
        "linear": linear,
        "info": info,
        "quiet": quiet,
        "debug": debug,
        "force": force,
        "help": help_,
        "version": version,
        "in_tracks": in_tracks,
        "in_fod": in_fod,
        "out_weights": out_weights,
    }
    if proc_mask is not None:
        params["proc_mask"] = proc_mask
    if act is not None:
        params["act"] = act
    if fd_thresh is not None:
        params["fd_thresh"] = fd_thresh
    if csv_ is not None:
        params["csv"] = csv_
    if out_mu is not None:
        params["out_mu"] = out_mu
    if out_coeffs is not None:
        params["out_coeffs"] = out_coeffs
    if reg_tikhonov is not None:
        params["reg_tikhonov"] = reg_tikhonov
    if reg_tv is not None:
        params["reg_tv"] = reg_tv
    if min_td_frac is not None:
        params["min_td_frac"] = min_td_frac
    if min_iters is not None:
        params["min_iters"] = min_iters
    if max_iters is not None:
        params["max_iters"] = max_iters
    if min_factor is not None:
        params["min_factor"] = min_factor
    if min_coeff is not None:
        params["min_coeff"] = min_coeff
    if max_factor is not None:
        params["max_factor"] = max_factor
    if max_coeff is not None:
        params["max_coeff"] = max_coeff
    if max_coeff_step is not None:
        params["max_coeff_step"] = max_coeff_step
    if min_cf_decrease is not None:
        params["min_cf_decrease"] = min_cf_decrease
    if nthreads is not None:
        params["nthreads"] = nthreads
    if config is not None:
        params["config"] = config
    return params


def tcksift2_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `Tcksift2ParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("proc_mask", None) is not None:
        if not isinstance(params["proc_mask"], (pathlib.Path, str)):
            raise StyxValidationError(f'`proc_mask` has the wrong type: Received `{type(params.get("proc_mask", None))}` expected `InputPathType | None`')
    if params.get("act", None) is not None:
        if not isinstance(params["act"], (pathlib.Path, str)):
            raise StyxValidationError(f'`act` has the wrong type: Received `{type(params.get("act", None))}` expected `InputPathType | None`')
    if params.get("fd_scale_gm", False) is None:
        raise StyxValidationError("`fd_scale_gm` must not be None")
    if not isinstance(params["fd_scale_gm"], bool):
        raise StyxValidationError(f'`fd_scale_gm` has the wrong type: Received `{type(params.get("fd_scale_gm", False))}` expected `bool`')
    if params.get("no_dilate_lut", False) is None:
        raise StyxValidationError("`no_dilate_lut` must not be None")
    if not isinstance(params["no_dilate_lut"], bool):
        raise StyxValidationError(f'`no_dilate_lut` has the wrong type: Received `{type(params.get("no_dilate_lut", False))}` expected `bool`')
    if params.get("make_null_lobes", False) is None:
        raise StyxValidationError("`make_null_lobes` must not be None")
    if not isinstance(params["make_null_lobes"], bool):
        raise StyxValidationError(f'`make_null_lobes` has the wrong type: Received `{type(params.get("make_null_lobes", False))}` expected `bool`')
    if params.get("remove_untracked", False) is None:
        raise StyxValidationError("`remove_untracked` must not be None")
    if not isinstance(params["remove_untracked"], bool):
        raise StyxValidationError(f'`remove_untracked` has the wrong type: Received `{type(params.get("remove_untracked", False))}` expected `bool`')
    if params.get("fd_thresh", None) is not None:
        if not isinstance(params["fd_thresh"], (float, int)):
            raise StyxValidationError(f'`fd_thresh` has the wrong type: Received `{type(params.get("fd_thresh", None))}` expected `float | None`')
    if params.get("csv", None) is not None:
        if not isinstance(params["csv"], str):
            raise StyxValidationError(f'`csv` has the wrong type: Received `{type(params.get("csv", None))}` expected `str | None`')
    if params.get("out_mu", None) is not None:
        if not isinstance(params["out_mu"], str):
            raise StyxValidationError(f'`out_mu` has the wrong type: Received `{type(params.get("out_mu", None))}` expected `str | None`')
    if params.get("output_debug", False) is None:
        raise StyxValidationError("`output_debug` must not be None")
    if not isinstance(params["output_debug"], bool):
        raise StyxValidationError(f'`output_debug` has the wrong type: Received `{type(params.get("output_debug", False))}` expected `bool`')
    if params.get("out_coeffs", None) is not None:
        if not isinstance(params["out_coeffs"], str):
            raise StyxValidationError(f'`out_coeffs` has the wrong type: Received `{type(params.get("out_coeffs", None))}` expected `str | None`')
    if params.get("reg_tikhonov", None) is not None:
        if not isinstance(params["reg_tikhonov"], (float, int)):
            raise StyxValidationError(f'`reg_tikhonov` has the wrong type: Received `{type(params.get("reg_tikhonov", None))}` expected `float | None`')
    if params.get("reg_tv", None) is not None:
        if not isinstance(params["reg_tv"], (float, int)):
            raise StyxValidationError(f'`reg_tv` has the wrong type: Received `{type(params.get("reg_tv", None))}` expected `float | None`')
    if params.get("min_td_frac", None) is not None:
        if not isinstance(params["min_td_frac"], (float, int)):
            raise StyxValidationError(f'`min_td_frac` has the wrong type: Received `{type(params.get("min_td_frac", None))}` expected `float | None`')
    if params.get("min_iters", None) is not None:
        if not isinstance(params["min_iters"], int):
            raise StyxValidationError(f'`min_iters` has the wrong type: Received `{type(params.get("min_iters", None))}` expected `int | None`')
    if params.get("max_iters", None) is not None:
        if not isinstance(params["max_iters"], int):
            raise StyxValidationError(f'`max_iters` has the wrong type: Received `{type(params.get("max_iters", None))}` expected `int | None`')
    if params.get("min_factor", None) is not None:
        if not isinstance(params["min_factor"], (float, int)):
            raise StyxValidationError(f'`min_factor` has the wrong type: Received `{type(params.get("min_factor", None))}` expected `float | None`')
    if params.get("min_coeff", None) is not None:
        if not isinstance(params["min_coeff"], (float, int)):
            raise StyxValidationError(f'`min_coeff` has the wrong type: Received `{type(params.get("min_coeff", None))}` expected `float | None`')
    if params.get("max_factor", None) is not None:
        if not isinstance(params["max_factor"], (float, int)):
            raise StyxValidationError(f'`max_factor` has the wrong type: Received `{type(params.get("max_factor", None))}` expected `float | None`')
    if params.get("max_coeff", None) is not None:
        if not isinstance(params["max_coeff"], (float, int)):
            raise StyxValidationError(f'`max_coeff` has the wrong type: Received `{type(params.get("max_coeff", None))}` expected `float | None`')
    if params.get("max_coeff_step", None) is not None:
        if not isinstance(params["max_coeff_step"], (float, int)):
            raise StyxValidationError(f'`max_coeff_step` has the wrong type: Received `{type(params.get("max_coeff_step", None))}` expected `float | None`')
    if params.get("min_cf_decrease", None) is not None:
        if not isinstance(params["min_cf_decrease"], (float, int)):
            raise StyxValidationError(f'`min_cf_decrease` has the wrong type: Received `{type(params.get("min_cf_decrease", None))}` expected `float | None`')
    if params.get("linear", False) is None:
        raise StyxValidationError("`linear` must not be None")
    if not isinstance(params["linear"], bool):
        raise StyxValidationError(f'`linear` has the wrong type: Received `{type(params.get("linear", False))}` expected `bool`')
    if params.get("info", False) is None:
        raise StyxValidationError("`info` must not be None")
    if not isinstance(params["info"], bool):
        raise StyxValidationError(f'`info` has the wrong type: Received `{type(params.get("info", False))}` expected `bool`')
    if params.get("quiet", False) is None:
        raise StyxValidationError("`quiet` must not be None")
    if not isinstance(params["quiet"], bool):
        raise StyxValidationError(f'`quiet` has the wrong type: Received `{type(params.get("quiet", False))}` expected `bool`')
    if params.get("debug", False) is None:
        raise StyxValidationError("`debug` must not be None")
    if not isinstance(params["debug"], bool):
        raise StyxValidationError(f'`debug` has the wrong type: Received `{type(params.get("debug", False))}` expected `bool`')
    if params.get("force", False) is None:
        raise StyxValidationError("`force` must not be None")
    if not isinstance(params["force"], bool):
        raise StyxValidationError(f'`force` has the wrong type: Received `{type(params.get("force", False))}` expected `bool`')
    if params.get("nthreads", None) is not None:
        if not isinstance(params["nthreads"], int):
            raise StyxValidationError(f'`nthreads` has the wrong type: Received `{type(params.get("nthreads", None))}` expected `int | None`')
    if params.get("config", None) is not None:
        if not isinstance(params["config"], list):
            raise StyxValidationError(f'`config` has the wrong type: Received `{type(params.get("config", None))}` expected `list[Tcksift2ConfigParamsDict] | None`')
        for e in params["config"]:
            tcksift2_config_validate(e)
    if params.get("help", False) is None:
        raise StyxValidationError("`help` must not be None")
    if not isinstance(params["help"], bool):
        raise StyxValidationError(f'`help` has the wrong type: Received `{type(params.get("help", False))}` expected `bool`')
    if params.get("version", False) is None:
        raise StyxValidationError("`version` must not be None")
    if not isinstance(params["version"], bool):
        raise StyxValidationError(f'`version` has the wrong type: Received `{type(params.get("version", False))}` expected `bool`')
    if params.get("in_tracks", None) is None:
        raise StyxValidationError("`in_tracks` must not be None")
    if not isinstance(params["in_tracks"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in_tracks` has the wrong type: Received `{type(params.get("in_tracks", None))}` expected `InputPathType`')
    if params.get("in_fod", None) is None:
        raise StyxValidationError("`in_fod` must not be None")
    if not isinstance(params["in_fod"], (pathlib.Path, str)):
        raise StyxValidationError(f'`in_fod` has the wrong type: Received `{type(params.get("in_fod", None))}` expected `InputPathType`')
    if params.get("out_weights", None) is None:
        raise StyxValidationError("`out_weights` must not be None")
    if not isinstance(params["out_weights"], str):
        raise StyxValidationError(f'`out_weights` has the wrong type: Received `{type(params.get("out_weights", None))}` expected `str`')


def tcksift2_cargs(
    params: Tcksift2ParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("tcksift2")
    if params.get("proc_mask", None) is not None:
        cargs.extend([
            "-proc_mask",
            execution.input_file(params.get("proc_mask", None))
        ])
    if params.get("act", None) is not None:
        cargs.extend([
            "-act",
            execution.input_file(params.get("act", None))
        ])
    if params.get("fd_scale_gm", False):
        cargs.append("-fd_scale_gm")
    if params.get("no_dilate_lut", False):
        cargs.append("-no_dilate_lut")
    if params.get("make_null_lobes", False):
        cargs.append("-make_null_lobes")
    if params.get("remove_untracked", False):
        cargs.append("-remove_untracked")
    if params.get("fd_thresh", None) is not None:
        cargs.extend([
            "-fd_thresh",
            str(params.get("fd_thresh", None))
        ])
    if params.get("csv", None) is not None:
        cargs.extend([
            "-csv",
            params.get("csv", None)
        ])
    if params.get("out_mu", None) is not None:
        cargs.extend([
            "-out_mu",
            params.get("out_mu", None)
        ])
    if params.get("output_debug", False):
        cargs.append("-output_debug")
    if params.get("out_coeffs", None) is not None:
        cargs.extend([
            "-out_coeffs",
            params.get("out_coeffs", None)
        ])
    if params.get("reg_tikhonov", None) is not None:
        cargs.extend([
            "-reg_tikhonov",
            str(params.get("reg_tikhonov", None))
        ])
    if params.get("reg_tv", None) is not None:
        cargs.extend([
            "-reg_tv",
            str(params.get("reg_tv", None))
        ])
    if params.get("min_td_frac", None) is not None:
        cargs.extend([
            "-min_td_frac",
            str(params.get("min_td_frac", None))
        ])
    if params.get("min_iters", None) is not None:
        cargs.extend([
            "-min_iters",
            str(params.get("min_iters", None))
        ])
    if params.get("max_iters", None) is not None:
        cargs.extend([
            "-max_iters",
            str(params.get("max_iters", None))
        ])
    if params.get("min_factor", None) is not None:
        cargs.extend([
            "-min_factor",
            str(params.get("min_factor", None))
        ])
    if params.get("min_coeff", None) is not None:
        cargs.extend([
            "-min_coeff",
            str(params.get("min_coeff", None))
        ])
    if params.get("max_factor", None) is not None:
        cargs.extend([
            "-max_factor",
            str(params.get("max_factor", None))
        ])
    if params.get("max_coeff", None) is not None:
        cargs.extend([
            "-max_coeff",
            str(params.get("max_coeff", None))
        ])
    if params.get("max_coeff_step", None) is not None:
        cargs.extend([
            "-max_coeff_step",
            str(params.get("max_coeff_step", None))
        ])
    if params.get("min_cf_decrease", None) is not None:
        cargs.extend([
            "-min_cf_decrease",
            str(params.get("min_cf_decrease", None))
        ])
    if params.get("linear", False):
        cargs.append("-linear")
    if params.get("info", False):
        cargs.append("-info")
    if params.get("quiet", False):
        cargs.append("-quiet")
    if params.get("debug", False):
        cargs.append("-debug")
    if params.get("force", False):
        cargs.append("-force")
    if params.get("nthreads", None) is not None:
        cargs.extend([
            "-nthreads",
            str(params.get("nthreads", None))
        ])
    if params.get("config", None) is not None:
        cargs.extend([a for c in [tcksift2_config_cargs(s, execution) for s in params.get("config", None)] for a in c])
    if params.get("help", False):
        cargs.append("-help")
    if params.get("version", False):
        cargs.append("-version")
    cargs.append(execution.input_file(params.get("in_tracks", None)))
    cargs.append(execution.input_file(params.get("in_fod", None)))
    cargs.append(params.get("out_weights", None))
    return cargs


def tcksift2_outputs(
    params: Tcksift2ParamsDict,
    execution: Execution,
) -> Tcksift2Outputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Tcksift2Outputs(
        root=execution.output_file("."),
        out_weights=execution.output_file(params.get("out_weights", None)),
        csv_=execution.output_file(params.get("csv", None)) if (params.get("csv") is not None) else None,
        out_mu=execution.output_file(params.get("out_mu", None)) if (params.get("out_mu") is not None) else None,
        out_coeffs=execution.output_file(params.get("out_coeffs", None)) if (params.get("out_coeffs") is not None) else None,
    )
    return ret


def tcksift2_execute(
    params: Tcksift2ParamsDict,
    runner: Runner | None = None,
) -> Tcksift2Outputs:
    """
    tcksift2
    
    Optimise per-streamline cross-section multipliers to match a whole-brain
    tractogram to fixel-wise fibre densities.
    
    
    
    References:
    
    Smith, R. E.; Tournier, J.-D.; Calamante, F. & Connelly, A. SIFT2: Enabling
    dense quantitative assessment of brain white matter connectivity using
    streamlines tractography. NeuroImage, 2015, 119, 338-351
    
    * If using the -linear option:
    Smith, RE; Raffelt, D; Tournier, J-D; Connelly, A. Quantitative Streamlines
    Tractography: Methods and Inter-Subject Normalisation. Open Science
    Framework, https://doi.org/10.31219/osf.io/c67kn.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tcksift2Outputs`).
    """
    tcksift2_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(TCKSIFT2_METADATA)
    params = execution.params(params)
    cargs = tcksift2_cargs(params, execution)
    ret = tcksift2_outputs(params, execution)
    execution.run(cargs)
    return ret


def tcksift2(
    in_tracks: InputPathType,
    in_fod: InputPathType,
    out_weights: str,
    proc_mask: InputPathType | None = None,
    act: InputPathType | None = None,
    fd_scale_gm: bool = False,
    no_dilate_lut: bool = False,
    make_null_lobes: bool = False,
    remove_untracked: bool = False,
    fd_thresh: float | None = None,
    csv_: str | None = None,
    out_mu: str | None = None,
    output_debug: bool = False,
    out_coeffs: str | None = None,
    reg_tikhonov: float | None = None,
    reg_tv: float | None = None,
    min_td_frac: float | None = None,
    min_iters: int | None = None,
    max_iters: int | None = None,
    min_factor: float | None = None,
    min_coeff: float | None = None,
    max_factor: float | None = None,
    max_coeff: float | None = None,
    max_coeff_step: float | None = None,
    min_cf_decrease: float | None = None,
    linear: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[Tcksift2ConfigParamsDict] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> Tcksift2Outputs:
    """
    tcksift2
    
    Optimise per-streamline cross-section multipliers to match a whole-brain
    tractogram to fixel-wise fibre densities.
    
    
    
    References:
    
    Smith, R. E.; Tournier, J.-D.; Calamante, F. & Connelly, A. SIFT2: Enabling
    dense quantitative assessment of brain white matter connectivity using
    streamlines tractography. NeuroImage, 2015, 119, 338-351
    
    * If using the -linear option:
    Smith, RE; Raffelt, D; Tournier, J-D; Connelly, A. Quantitative Streamlines
    Tractography: Methods and Inter-Subject Normalisation. Open Science
    Framework, https://doi.org/10.31219/osf.io/c67kn.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        in_tracks: the input track file.
        in_fod: input image containing the spherical harmonics of the fibre\
            orientation distributions.
        out_weights: output text file containing the weighting factor for each\
            streamline.
        proc_mask: provide an image containing the processing mask weights for\
            the model; image spatial dimensions must match the fixel image.
        act: use an ACT five-tissue-type segmented anatomical image to derive\
            the processing mask.
        fd_scale_gm: provide this option (in conjunction with -act) to\
            heuristically downsize the fibre density estimates based on the\
            presence of GM in the voxel. This can assist in reducing tissue\
            interface effects when using a single-tissue deconvolution algorithm.
        no_dilate_lut: do NOT dilate FOD lobe lookup tables; only map\
            streamlines to FOD lobes if the precise tangent lies within the angular\
            spread of that lobe.
        make_null_lobes: add an additional FOD lobe to each voxel, with zero\
            integral, that covers all directions with zero / negative FOD\
            amplitudes.
        remove_untracked: remove FOD lobes that do not have any streamline\
            density attributed to them; this improves filtering slightly, at the\
            expense of longer computation time (and you can no longer do\
            quantitative comparisons between reconstructions if this is enabled).
        fd_thresh: fibre density threshold; exclude an FOD lobe from filtering\
            processing if its integral is less than this amount (streamlines will\
            still be mapped to it, but it will not contribute to the cost function\
            or the filtering).
        csv_: output statistics of execution per iteration to a .csv file.
        out_mu: output the final value of SIFT proportionality coefficient mu\
            to a text file.
        output_debug: provide various output images for assessing & debugging\
            performance etc.
        out_coeffs: output text file containing the weighting coefficient for\
            each streamline.
        reg_tikhonov: provide coefficient for regularising streamline weighting\
            coefficients (Tikhonov regularisation) (default: 0).
        reg_tv: provide coefficient for regularising variance of streamline\
            weighting coefficient to fixels along its length (Total Variation\
            regularisation) (default: 0.1).
        min_td_frac: minimum fraction of the FOD integral reconstructed by\
            streamlines; if the reconstructed streamline density is below this\
            fraction, the fixel is excluded from optimisation (default: 0.1).
        min_iters: minimum number of iterations to run before testing for\
            convergence; this can prevent premature termination at early iterations\
            if the cost function increases slightly (default: 10).
        max_iters: maximum number of iterations to run before terminating\
            program.
        min_factor: minimum weighting factor for an individual streamline; if\
            the factor falls below this number the streamline will be rejected\
            entirely (factor set to zero) (default: 0).
        min_coeff: minimum weighting coefficient for an individual streamline;\
            similar to the '-min_factor' option, but using the exponential\
            coefficient basis of the SIFT2 model; these parameters are related as:\
            factor = e^(coeff). Note that the -min_factor and -min_coeff options\
            are mutually exclusive - you can only provide one. (default: -inf).
        max_factor: maximum weighting factor that can be assigned to any one\
            streamline (default: inf).
        max_coeff: maximum weighting coefficient for an individual streamline;\
            similar to the '-max_factor' option, but using the exponential\
            coefficient basis of the SIFT2 model; these parameters are related as:\
            factor = e^(coeff). Note that the -max_factor and -max_coeff options\
            are mutually exclusive - you can only provide one. (default: inf).
        max_coeff_step: maximum change to a streamline's weighting coefficient\
            in a single iteration (default: 1).
        min_cf_decrease: minimum decrease in the cost function (as a fraction\
            of the initial value) that must occur each iteration for the algorithm\
            to continue (default: 2.5e-05).
        linear: perform a linear estimation of streamline weights, rather than\
            the standard non-linear optimisation (typically does not provide as\
            accurate a model fit; but only requires a single pass).
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Tcksift2Outputs`).
    """
    params = tcksift2_params(
        proc_mask=proc_mask,
        act=act,
        fd_scale_gm=fd_scale_gm,
        no_dilate_lut=no_dilate_lut,
        make_null_lobes=make_null_lobes,
        remove_untracked=remove_untracked,
        fd_thresh=fd_thresh,
        csv_=csv_,
        out_mu=out_mu,
        output_debug=output_debug,
        out_coeffs=out_coeffs,
        reg_tikhonov=reg_tikhonov,
        reg_tv=reg_tv,
        min_td_frac=min_td_frac,
        min_iters=min_iters,
        max_iters=max_iters,
        min_factor=min_factor,
        min_coeff=min_coeff,
        max_factor=max_factor,
        max_coeff=max_coeff,
        max_coeff_step=max_coeff_step,
        min_cf_decrease=min_cf_decrease,
        linear=linear,
        info=info,
        quiet=quiet,
        debug=debug,
        force=force,
        nthreads=nthreads,
        config=config,
        help_=help_,
        version=version,
        in_tracks=in_tracks,
        in_fod=in_fod,
        out_weights=out_weights,
    )
    return tcksift2_execute(params, runner)


__all__ = [
    "TCKSIFT2_METADATA",
    "Tcksift2ConfigParamsDict",
    "Tcksift2ConfigParamsDictTagged",
    "Tcksift2Outputs",
    "Tcksift2ParamsDict",
    "Tcksift2ParamsDictTagged",
    "tcksift2",
    "tcksift2_config",
    "tcksift2_execute",
    "tcksift2_params",
]
